#!/usr/bin/python3

import os
from PyQt6.QtWidgets import QTabWidget, QAbstractItemView

from ..player.playlist import Playlist
from ..ui.library.library import LibraryWidget
from ..ui.playlist_view import PlaylistView


class Library:
    """
    Class that manages and syncs the library and library gui dock widgets.
    """

    def __init__(self, app):
        self.app = app

        self.main_library_widget = LibraryWidget(self)
        self.library_widgets = [self.main_library_widget]

        self.loaded_tracks = {}  # dict of {track path: track}

        self.playlists = []
        self.temporary_playlist = None

        self.artist_playlists = []

    def load(self):
        self.load_playlists()

    def load_playlists(self):
        path_playlists = os.path.expanduser(f"{self.app.settings.library_path}/playlists")

        if not os.path.exists(path_playlists):
            os.makedirs(path_playlists)

            self.load_playlist_views()

            return

        folder_content = os.listdir(path_playlists)

        for file_name in folder_content:
            if file_name.endswith(".m3u"):
                path = f"{path_playlists}/{file_name}"

                playlist = Playlist(self.app, file_name.replace("_", " ").split(".")[0])
                self.playlists.append(playlist)

                if playlist.title == "Temporary Playlist":
                    self.temporary_playlist = playlist

    def load_playlist_views(self):
        # create views for each dock and playlist

        for library_widget in self.library_widgets:
            playlist_tabs: QTabWidget = library_widget.playlist_tabs

            # create view for each playlist
            for playlist in self.playlists:
                if id(library_widget) in playlist.views:  # view already exists
                    continue

                playlist_view = PlaylistView(playlist, library_widget)
                playlist_tabs.addTab(playlist_view, playlist.title)

                if playlist.path == self.app.settings.latest_playlist:  # start with latest playlist opened and loaded
                    playlist_tabs.setCurrentIndex(playlist_tabs.count() - 1)

                    playlist.load()

        if self.app.settings.load_on_start:
            for playlist in self.playlists:
                playlist.load()

    def on_exit(self, event):
        for playlist in self.playlists:
            if playlist.loaded:  # only save loaded playlists, unloaded are empty
                playlist.save()

        if self.app.player.current_playlist is not None:
            self.app.settings.latest_playlist = self.app.player.current_playlist.path

        print(self.loaded_tracks)

    def new_playlist(self):
        playlist = Playlist(self.app, self.app.utils.unique_name("New Playlist"))
        playlist.loaded = True

        self.playlists.append(playlist)

        for library_widget in self.library_widgets:
            playlist_tabs: QTabWidget = library_widget.playlist_tabs

            playlist_view = PlaylistView(playlist, library_widget, playlist_tabs)
            playlist_view.setDragDropMode(QAbstractItemView.DragDropMode.InternalMove)  # enable drag n drop

            playlist_tabs.addTab(playlist_view, playlist.title)

    def replace_temporary_playlist(self, replace: Playlist):
        if self.temporary_playlist is not None:
            self.temporary_playlist.delete()

        if not replace in self.playlists:
            self.playlists.append(replace)

        self.temporary_playlist = replace

    def open_tracks(self, tracks: list[str]):
        playlist = Playlist(self.app, "Temporary Playlist", tracks)

        self.replace_temporary_playlist(playlist)

        self.load_playlist_views()

        playlist.load()

    def open_playlist(self, playlist_path: str):
        playlist = Playlist(self.app, "Temporary Playlist", playlist_path)

        self.replace_temporary_playlist(playlist)

        self.load_playlist_views()

        playlist.load()

    def import_playlist(self, playlist_path: str):
        self.open_playlist(playlist_path)

    def loaded_track(self, track_path: str):
        """
        Returns either a loaded track with the given path or None if there is none.
        """

        if track_path in self.loaded_tracks:
            return self.loaded_tracks[track_path]

